/**
 * Copyright (c) 2021 Dominic Msters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "game.h"

game_t * gameInit(platform_t *platform) {
  // Create the game
  game_t *game = malloc(sizeof(game_t));
  if(game == NULL) return NULL;

  // Load the game engine
  game->engine = engineInit(platform, GAME_NAME, GAME_INPUT_COUNT);
  if(game->engine == NULL) {
    free(game);
    return NULL;
  }

  // Load the Shader
  game->shader = assetShaderLoad("shaders/test.vert", "shaders/test.frag");
  
  // Prepare the camera.
  game->camera = cameraCreate();
  uint32_t d = 10;
  cameraLookAt(game->camera,  d, d, d,  0, 0, 0);
  cameraPerspective(game->camera, 45.0f, 1920.0f/1080.0f, 0.5f, 500.0f);

  // Load the world
  game->world = worldLoad("testworld/");

  // Pass to the main game to handle
  return game;
}

uint32_t gameUpdate(game_t *game) {
  uint32_t result;

  // Update the engine.
  result = engineUpdate(game->engine);
  if(result != 0) return result;

  // Prepare for rendering
  shaderUse(game->shader);
  shaderUseCamera(game->shader, game->camera);
  worldRender(game->world, game->shader, game->camera);

  return 0;
}

void gameDispose(game_t *game) {
  worldDispose(game->world);
  shaderDipose(game->shader);
  cameraDispose(game->camera);
  engineDispose(game->engine);
  free(game);
}

engine_t * gameGetEngine(game_t *game) {
  return game->engine;
}