/**
 * Copyright (c) 2021 Dominic Msters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#pragma once
#include <stdbool.h>
#include "../engine.h"
#include "../platform.h"
#include "../display/camera.h"
#include "../display/shader.h"
#include "../world/world.h"

/////////////////////////////////// CONSTANTS //////////////////////////////////
/** Name of the Game */
#define GAME_NAME "Dawn Game"

/** Inputs */
#define GAME_INPUT_UP          (inputbind_t)0x01
#define GAME_INPUT_DOWN        (inputbind_t)0x02
#define GAME_INPUT_LEFT        (inputbind_t)0x03
#define GAME_INPUT_RIGHT       (inputbind_t)0x04

#define GAME_INPUT_COUNT 5

/////////////////////////////// TYPE DEFINITIONS ///////////////////////////////
/** Information about the current game context. */
typedef struct {
  /** The engine context for the game */
  engine_t *engine;

  /** Rendering items */
  camera_t *camera;
  shader_t *shader;
  world_t *world;
} game_t;

/**
 * Initialize the game context.
 * 
 * @return The game instance context.
 */
game_t * gameInit(platform_t *platform);

/**
 * Start the main game loop.
 * 
 * @param game The game to start the loop for.
 * @return Refer to engineUpdate. 0 for loop continue, 1 for safe exit.
 */
uint32_t gameUpdate(game_t *game);

/**
 * Cleanup a previously constructed.
 * @param game The game to cleanup.
 */
void gameDispose(game_t *game);