// Copyright (c) 2021 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include <malloc.h>
#include <stdint.h>
#include "worldtypes.h"
#include "tile.h"
#include "chunk.h"
#include "../display/shader.h"
#include "../display/camera.h"
#include "../display/primitive.h"
#include "../display/tileset.h"
#include "../display/texture.h"
#include "../file/asset.h"
#include "../util/string.h"
#include "../util/math.h"

/** Token in the world data file to split on. */
#define WORLD_LOAD_TOKEN ";"

/**
 * Create a world object.
 * 
 * @returns The newly created world.
 */
world_t * worldLoad(char *fileName);

/**
 * Render a world object to the graphics device.
 * 
 * @param world The world to render.
 * @param shader The shader to render to.
 */
void worldRender(world_t *world, shader_t *shader, camera_t *camera);

/**
 * Cleans up a previously created world.
 * 
 * @param world World to cleanup.
 */
void worldDispose(world_t *world);

/**
 * Shift the world chunk list along a set of axis (in absolute space).
 * 
 * @param world World to shift.
 * @param x X movement to shift chunk along.
 * @param y Y movement to shift chunk along.
 * @param z Z movement to shift chunk along.
 */
void worldShift(world_t *world, int32_t x, int32_t y, int32_t z);

/**
 * Align the world chunk list (in absolute space).
 * 
 * @param world World to align.
 * @param x X movement to shift chunk along.
 * @param y Y movement to shift chunk along.
 * @param z Z movement to shift chunk along.
 */
void worldAlign(world_t *world, int32_t x, int32_t y, int32_t z);